"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parser_1 = require("../../parser");
const css_nodes_1 = require("../../utils/css-nodes");
function hasInvalidEOF(node) {
    var _a;
    const body = node.templateBody;
    if (((_a = body) === null || _a === void 0 ? void 0 : _a.errors) == null) {
        return false;
    }
    return body.errors.some(error => typeof error.code === "string" && error.code.startsWith("eof-"));
}
function getStyleElements(context) {
    let document = null;
    if (context.parserServices.getDocumentFragment) {
        document = context.parserServices.getDocumentFragment();
    }
    else {
        const sourceCode = context.getSourceCode();
        const { ast } = sourceCode;
        const templateBody = ast.templateBody;
        if (templateBody) {
            document = templateBody.parent;
        }
    }
    if (document) {
        return document.children
            .filter(isVElement)
            .filter(element => element.name === "style");
    }
    return [];
}
function isScoped(style) {
    const { startTag } = style;
    return startTag.attributes.some(attr => attr.key.name === "scoped");
}
function getLang(style) {
    var _a, _b;
    const { startTag } = style;
    const lang = startTag.attributes.find(attr => attr.key.name === "lang") || null;
    return (((_a = lang) === null || _a === void 0 ? void 0 : _a.type) === "VAttribute" &&
        ((_b = lang.value) === null || _b === void 0 ? void 0 : _b.type) === "VLiteral" &&
        lang.value.value);
}
class StyleContext {
    constructor(style, sourceCode) {
        this.styleElement = style;
        this.sourceCode = sourceCode;
        const { startTag, endTag } = style;
        this.invalid = endTag == null || hasInvalidEOF(sourceCode.ast);
        this.scoped = Boolean(style && isScoped(style));
        this.lang = ((style && getLang(style)) || "css").toLowerCase();
        if (!this.invalid && endTag != null) {
            this.cssText = sourceCode.text.slice(startTag.range[1], endTag.range[0]);
            this.cssNode = parser_1.parse(sourceCode, startTag.loc.end, this.cssText, this.lang);
        }
        else {
            this.cssText = null;
            this.cssNode = null;
        }
    }
    traverseNodes(visitor) {
        if (this.cssNode) {
            traverseNodes(this.cssNode, visitor);
        }
    }
}
exports.StyleContext = StyleContext;
function traverseNodes(node, visitor) {
    visitor.break = false;
    visitor.enterNode(node);
    if (visitor.exit || visitor.break) {
        return;
    }
    if (css_nodes_1.isVCSSContainerNode(node)) {
        for (const child of node.nodes) {
            traverseNodes(child, visitor);
            if (visitor.break) {
                break;
            }
            if (visitor.exit) {
                return;
            }
        }
    }
    visitor.leaveNode(node);
}
function createStyleContexts(context) {
    const sourceCode = context.getSourceCode();
    const styles = getStyleElements(context);
    return styles.map(style => new StyleContext(style, sourceCode));
}
exports.createStyleContexts = createStyleContexts;
function isVElement(node) {
    var _a;
    return ((_a = node) === null || _a === void 0 ? void 0 : _a.type) === "VElement";
}
